import * as Electron from 'electron';

import log from 'electron-log';

import { AppChannel } from '@ucap/electron-core';
import { AppLoader } from '../common/app/app-loader';
import { AppSettings } from '../common/app/decorators/app-settings';
import { On } from '../common/app/decorators/on';

import { AppWindow } from './app.window';
import { BrowserWindowRegistry } from '../common/browser-window/registries/browser-window.registry';

@AppSettings({
  bootstrap: AppWindow
})
export class App extends AppLoader {
  constructor() {
    super();
  }

  @On(AppChannel.SecondInstance)
  onSecondInstance(event: Event, argv: string[], workingDirectory: string) {
    log.info('AppChannel.SecondInstance');
  }

  @On(AppChannel.Ready)
  onReady(launchInfo: any) {
    log.info('AppChannel.Ready');

    this.createMainWindow();
  }

  @On(AppChannel.Activate)
  onActivate(event: Event, hasVisibleWindows: boolean) {
    log.info('AppChannel.Activate');
  }

  @On(AppChannel.WindowAllClosed)
  onWindowAllClosed() {
    log.info('AppChannel.WindowAllClosed');
  }

  private createMainWindow() {
    const bootstrap = this.settings.get('bootstrap');

    const provider = BrowserWindowRegistry.get(bootstrap);
    const i = provider.useFactory(this.electronApp);
  }

  private createTrayIcon() {}
}
