import * as Electron from 'electron';
import log from 'electron-log';
import * as windowStateKeeper from 'electron-window-state';

import { BrowserWindow } from '../common/browser-window/decorators/browser-window';
import { On } from '../common/browser-window/decorators/on';
import { BrowserWindowChannel, AppChannel } from '@ucap/electron-core';

const MIN_WIDTH = 700;
const MIN_HEIGHT = 600;
const DEFAULT_WIDTH = 1160;
const DEFAULT_HEIGHT = 800;

let savedWindowState: windowStateKeeper.State;

@BrowserWindow({
  constructorOptions: {
    minWidth: MIN_WIDTH,
    minHeight: MIN_HEIGHT,
    center: true,
    backgroundColor: '#fff',
    webPreferences: {
      // Disable auxclick event
      // See https://developers.google.com/web/updates/2016/10/auxclick
      disableBlinkFeatures: 'Auxclick',
      // Enable, among other things, the ResizeObserver
      experimentalFeatures: true,
      nodeIntegration: true
    },
    acceptFirstMouse: true
  },
  beforeConstructor: constructorOptions => {
    savedWindowState = windowStateKeeper({
      defaultWidth: DEFAULT_WIDTH,
      defaultHeight: DEFAULT_HEIGHT
    });

    return {
      ...constructorOptions,
      x: savedWindowState.x,
      y: savedWindowState.y,
      width: savedWindowState.width,
      height: savedWindowState.height
    };
  }
})
export class AppWindow {
  constructor(
    private app: Electron.App,
    private window: Electron.BrowserWindow
  ) {
    savedWindowState.manage(this.window);

    this.attachHandlers();
  }

  @On(BrowserWindowChannel.ReadyToShow)
  onReadyToShow() {
    log.info('BrowserWindowChannel.ReadyToShow');
  }

  @On(BrowserWindowChannel.Close)
  onClose(event: Event) {
    log.info('BrowserWindowChannel.Close');
  }

  @On(BrowserWindowChannel.Focus)
  onFocus() {
    log.info('BrowserWindowChannel.Focus');
  }

  @On(BrowserWindowChannel.Blur)
  onBlur() {
    log.info('BrowserWindowChannel.Blur');
  }

  @On(BrowserWindowChannel.Minimize)
  onMinimize() {
    log.info('BrowserWindowChannel.Minimize');
  }

  @On(BrowserWindowChannel.Maximize)
  onMaximize() {
    log.info('BrowserWindowChannel.Maximize');
  }

  @On(BrowserWindowChannel.Unmaximize)
  onUnmaximize() {
    log.info('BrowserWindowChannel.Unmaximize');
  }

  @On(BrowserWindowChannel.Closed)
  onClosed() {
    log.info('BrowserWindowChannel.Closed');
  }

  private attachHandlers() {
    this.attachAppHandlers();
  }

  private attachAppHandlers() {
    this.app.on(AppChannel.BeforeQuit, (event: Electron.Event) => {
      log.info('AppChannel.BeforeQuit');
    });
  }
}
